/*********************************************************************
 **
 ** AirTraf - Utility to measure traffic in the wireless segment of
 **           802.11b network.  Discovers connected nodes, then tracks
 **           incoming/outgoing traffic for each node.  Shows the
 **           current bandwidth usage, as well as total packet count,
 **           inc/outgoing for each node, and tracks specific
 **           management frames sent from the AP.
 **
 **           This utility is designed to work with the AirTraf sniff
 **           server, which when combined, provides an extremely
 **           focused point of administration, which allows collection
 **           of data from multiple sniff servers to be viewed via a
 **           single web interface.
 **
 *********************************************************************
 *
 * There are many programs currently out there that does performance
 * monitoring.  However, this program is unique in that it isolates the
 * traffic to nodes within the wireless segment, regardless of the
 * fact that the wireless segment is connected to the rest of the
 * network or not.
 *
 * It will currently be used to analyze packet information upto layer
 * 2 of the OSI model, however, it can be extended beyond if proven
 * desirable.
 *
 * This utility is written for use with IEEE 802.11 adapters based
 * on Intersil's PRISM II chipset (PCMCIA).  However currently, the
 * limitation of Prism2 drivers limit the sniffing of channel to
 * single channels.  It is also equipped to work with Cisco
 * Aironet chipset, which offers an advantage of sniffing on all
 * channels at once.
 *
 * The linux driver for these cards can be found on www.linux-wlan.com,
 * as well as from www.sourceforge.net/projects/airtraf for the
 * modified aironet drivers.
 *
 * It has been verified with a LinkSys WPC11 IEEE 802.11 adapter.
 * It has also been verified with Cisco Aironet 340 adapter.
 * 
 * Copyright (c)2001,2002 by Elixar.net  Durham, NC., all rights reserved.
 *
 * Comments/Bug reports should be sent to: Peter K. Lee (saint@elixar.net)
 * 
 *********************************************************************/

/*=============================================================*/
/* System Includes */

#include <stdio.h>
#include <stdlib.h> // for atoi()
#include <mysql.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <asm/types.h>
#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>
#include <features.h>
#include <pthread.h>
#include "getopt.h"

#include "definition.h"
#include "poll_include.h"

/*=============================================================*/
/* Local Static Definitions */

#define def_host_name    NULL // host to connect to (default = localhost)
#define def_user_name    NULL // user name (default = your login name)
#define def_password     NULL // password (default = none)
#define def_port_num     0    // use default port
#define def_socket_name  NULL // use default socket name
#define def_db_name      NULL // database to use (default = none)
#define def_logging_mode DISABLED // enable logging
#define def_logfile     "log.polling_status"

#define MAX_QUERY        2000
#define SYNCH_CMD        1
#define GET_DATA_CMD     2

/*=============================================================*/
/* Local Global Variables */

char *groups[] = { "client", NULL };

MYSQL *conn; // pointer to connection handler

int stop_scanning = 0;
__u8 llc[7] = { 0xaa, 0xaa, 0x03, 0x00, 0x00, 0x00, 0x08 };
__u8 bcast[6] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };

/*=============================================================*/
/* Local Structs */

struct option long_options[] =
  {
    {"host",           required_argument, NULL, 'h'},
    {"user",           required_argument, NULL, 'u'},
    {"password",       optional_argument, NULL, 'p'},
    {"port",           required_argument, NULL, 'P'},
    {"socket",         required_argument, NULL, 'S'},
    {"logfile",        optional_argument, NULL, 'L'},
    {"help",           no_argument,       NULL, 'H'},
    { 0, 0, 0, 0}
  };


/*=============================================================*/
/* Function Prototypes */


/*=============================================================*/
/* Function Definitions */

static void usage (const char *pname)
{ 
  fprintf(stderr, "AirTraf: a wireless (802.11) network performance analyzer. by Peter K. Lee\n");
  fprintf(stderr, "\nUsage:\n\t%s [-h <host>] [-u <user>] [-p <password>] [-P <port>] [-S <socket>] [-L <logfile>] [-H] dbname\n", pname);
  fprintf(stderr, "\n-h : host to connect to *default:localhost*\n");
  fprintf(stderr, "-u : username to connect as *default:login*\n");
  fprintf(stderr, "-p : password *default:none* (optional)\n");
  fprintf(stderr, "-P : port *default:default port* \n");
  fprintf(stderr, "-S : socket *default:default socket* \n");
  fprintf(stderr, "-L : enable logging mode *default:OFF (argument allows specification of log file name) \n");
  fprintf(stderr, "-h : view this usage message\n\n");
  exit(1);
}

void stop_signal ()
{
  fprintf (stderr,"Received CTRL-C - scanning aborted\n");
  stop_scanning = 1;
}
 
/*=============================================================*/
/* Main Program */

int main(int argc, char **argv)
{
  /** program name **/
  char *pname = argv[0];
  
  /** mysql db connection stuff **/
  char *host_name = def_host_name;
  char *user_name = def_user_name;
  char *password = def_password;
  unsigned int port_num = def_port_num;
  char *socket_name = def_socket_name;
  char *db_name = def_db_name;
  char *logfile = def_logfile;
  int logging_mode = def_logging_mode;
  
  char passbuf[100];
  char lognamebuf[100];
  char temp_db_name[20];

  /** mysql query stuff **/
  char query[MAX_QUERY];
  MYSQL_RES *res_set;
  MYSQL_ROW row;
  char ip_addr[20];

  /** timing stuff **/
  struct timeval tv_old;
  struct timeval tv_new;
  float t_diff;
  int first_instance = 1;
  
  int ask_password = 0;  
  int c, option_index = 0;
  
//  uid_t runuid;
  int thread_count;
  
//  if((runuid = getuid()) != 0){
//    fprintf(stderr,"AirTRAF: a wireless (802.11) performance monitor utility. by Peter K. Lee\n");
//    fprintf(stderr,"\nThis program requires the use of priviledged operations and as such must be run as root.\n");
//    exit(1);
//  }

  if (argc < 2){
    usage(pname);
  }

  /** load defaults, a mysql initializer **/
/*   my_init(); */
/*   load_defaults("my", groups, &argc, &argv); */

  /** parse command-line arguments **/
  while ((c = getopt_long(argc, argv, "h:p::u:P:S:L::H", long_options,
			   &option_index)) != EOF){
    switch(c)
      {
      case 'h':
	host_name = optarg;
	break;
      case 'u':
	user_name = optarg;
	break;
      case 'p':
	if (!optarg)
	  ask_password = 1;
	else{
	  (void) strncpy (passbuf, optarg, sizeof(passbuf) -1);
	  passbuf[sizeof(passbuf)-1] = '\0';
	  password = passbuf;
	  while (*optarg)
	    *optarg++ = ' ';
	}
	break;
      case 'P':
	port_num = (unsigned int) atoi (optarg);
	break;
      case 'S':
	socket_name = optarg;
	break;
      case 'L':
	logging_mode = ENABLED;
	/* if there's optional argument */
	if (optarg){    
	  (void) strncpy(lognamebuf, optarg, sizeof(lognamebuf) -1);
	  lognamebuf[sizeof(lognamebuf)-1] = '\0';
	  logfile = lognamebuf;
	  while (*optarg){
	    *optarg++ = ' ';
	  }
	}
	break;
      case 'H':
	usage(pname);
	break;
      case '?':
	fprintf(stderr,"\nInvalid option or missing parameter, use airtraf-p -H for help\n\n");
	exit(1);
      }
  }

  argc -= optind; // advance past the arguments that were processed
  argv += optind; // by getopt_long()

  if (argc > 0){
    db_name = argv[0];
    --argc; ++argv;
  }
  else{
    sprintf(temp_db_name, "airtraf");
    db_name = temp_db_name;
  }

  if (ask_password){
    password = get_tty_password(NULL);
  }

  fprintf(stderr,"-----------------------------------------\n");
  fprintf(stderr,"Airtraf %s (c)2001,2002 Elixar.net\n", VERSION_INFO);
  fprintf(stderr,"Mode: polling server\n");
  fprintf(stderr,"Author: Peter K. Lee  All Rights Reserved\n");
  fprintf(stderr,"-----------------------------------------\n");

  fprintf(stderr,"\nconnecting to database... ");
  /** connect to the database **/
  conn = db_connect(host_name, user_name, password, db_name, port_num, socket_name, 0);
  if (conn == NULL) {
    fprintf(stderr,"failed!\n");
    exit(1);
  }
  else{
    fprintf(stderr,"OK\n");
  }

  /** increase priority of this process **/
  setpriority (PRIO_PROCESS, 0 , -20);
  
  /** catch SIGINT signal **/
  signal (SIGINT, stop_signal);  

  /** initialize lock for thread synchronization **/
  pthread_mutex_init(&ready_lock, NULL);
  pthread_cond_init(&ready_cond, NULL);

  /** initialize timer **/
  gettimeofday(&tv_old, NULL);
  
  /**
   * until the program is terminated, repeat these steps:
   * 1) see if its been 5 mins since last poll.
   * 2) query db to get latest sniff_info data (ip of servers)
   * 3) for each ip retrieved, start up the data collection thread(s)
   * 4) upon completion, free up the return_set, and wait again...
   **/
  while(!stop_scanning){
    
    void * vptr;
    void * row_ptr;

    int print_data_msg = 0;
    unsigned long row_count;
    
    gettimeofday(&tv_new, NULL);
    t_diff = get_time_diff(&tv_new, &tv_old);

    /**
     * unless its first instance, or t_diff is greater than 5 mins,
     * then just skip back to the top of the loop.
     **/
    if (!first_instance && (t_diff < 60 * 1 )){
      if (DEBUG) fprintf(stderr,"main waiting... elapsed: %0.2f\n", t_diff);
      sleep(15);
      if ((nthreads == 0) && (print_data_msg)){
	fprintf(stderr,"data acquisition complete!\n");
	print_data_msg = 0;
      }
      continue; // do loop all over...
    }
    if (first_instance){
      command = SYNCH_CMD;
      first_instance = 0;
    }
    else{
      command = GET_DATA_CMD;
    }
    tv_old = tv_new;
    
    /** construct query **/
    memset(query, 0, MAX_QUERY);
    snprintf(query, MAX_QUERY, "SELECT ip_address FROM sniff_info");

    fprintf(stderr, "---------------------------------\n");
    fprintf(stderr, "getting data from db... ");
    /** issue query **/
    if (db_query(conn, query) < 0){
      print_error(conn, "failed!");
      return (1); // just return for now...
    }

    if ((row_count = db_get_result(conn, &vptr)) < 1){
      print_error(conn, "failed!");
      return (1); // again, return for now...
    }
    fprintf(stderr, "OK\ngenerating %ld data gathering threads... \n",
	    row_count);
    fprintf(stderr, "---------------------------------\n");
    res_set = (MYSQL_RES *) vptr;

    thread_count = 0;
    nthreads = 0;
    while (db_get_row(res_set, &row_ptr) == RESULT_OK){
      thread_count++;
      row = (MYSQL_ROW) row_ptr;
      memset(ip_addr, 0, 20);
      snprintf(ip_addr, 20, "%s", row[0]);

      /** get lock to make sure we're waiting before the thread
	  broadcasts its availability... **/
      pthread_mutex_lock(&ready_lock);
      
      if (DEBUG) fprintf(stderr,"making thread for: %s\n", ip_addr);
      thread_make(thread_count, ip_addr);
      pthread_cond_wait(&ready_cond, &ready_lock);  // wait before
						    // overwriting ip
      pthread_mutex_unlock(&ready_lock);
    }
    mysql_free_result(res_set);
    print_data_msg = 1;
  }
  
  printf("done!\n");
  return (0);
}


