/*********************************************************************
 **
 ** AirTraf - Utility to measure traffic in the wireless segment of
 **           802.11b network.  Discovers connected nodes, then tracks
 **           incoming/outgoing traffic for each node.  Shows the
 **           current bandwidth usage, as well as total packet count,
 **           inc/outgoing for each node, and tracks specific
 **           management frames sent from the AP.
 **
 ** init-airtraf.c :
 **
 **           This utility is for initialization of all databases that
 **           the polling server requires for proper operation.
 **           It also imports initial data from configuration file, as
 **           well as has option of clearing out all database
 **           information from the database for troubleshooting
 **           purposes.
 **
 *********************************************************************
 *
 * There are many programs currently out there that does performance
 * monitoring.  However, this program is unique in that it isolates the
 * traffic to nodes within the wireless segment, regardless of the
 * fact that the wireless segment is connected to the rest of the
 * network or not.
 *
 * It will currently be used to analyze packet information upto layer
 * 2 of the OSI model, however, it can be extended beyond if proven
 * desirable.
 *
 * This utility is written for use with IEEE 802.11 adapters based
 * on Intersil's PRISM II chipset (PCMCIA).  However currently, the
 * limitation of Prism2 drivers limit the sniffing of channel to
 * single channels.  It is also equipped to work with Cisco
 * Aironet chipset, which offers an advantage of sniffing on all
 * channels at once.
 *
 * The linux driver for these cards can be found on www.linux-wlan.com,
 * as well as from www.sourceforge.net/projects/airtraf for the
 * modified aironet drivers.
 *
 * It has been verified with a LinkSys WPC11 IEEE 802.11 adapter.
 * It has also been verified with Cisco Aironet 340 adapter.
 * 
 * Copyright (c)2001,2002 by Elixar.net  Durham, NC., all rights reserved.
 *
 * Comments/Bug reports should be sent to: Peter K. Lee (pkl@duke.edu)
 * 
 *********************************************************************/

/*=============================================================*/
/* System Includes */

#include <stdio.h>
#include <stdlib.h> // for atoi()
#include <mysql.h>
#include <string.h>

#include "db_utils.h"
#include "getopt.h"

/*=============================================================*/
/* Local Static Definitions */

#define def_host_name    NULL // host to connect to (default = localhost)
#define def_user_name    NULL // user name (default = your login name)
#define def_password     NULL // password (default = none)
#define def_port_num     0    // use default port
#define def_socket_name  NULL // use default socket name
#define def_db_name      NULL // database to use (default = none)

#define IMPORT           1
#define RESET            2
#define MAX_QUERY        2000

/*=============================================================*/
/* Local Global Variables */

MYSQL * conn;

/*=============================================================*/
/* Local Structs */

struct option long_options[] =
  {
    {"host",           required_argument, NULL, 'h'},
    {"user",           required_argument, NULL, 'u'},
    {"password",       optional_argument, NULL, 'p'},
    {"port",           required_argument, NULL, 'P'},
    {"socket",         required_argument, NULL, 'S'},
    {"import",         required_argument, NULL, 'I'},
    {"reset",          no_argument,       NULL, 'R'},
    {"help",           no_argument,       NULL, 'H'},
    { 0, 0, 0, 0}
  };

/*=============================================================*/
/* Function Definitions */

static void usage (const char *pname)
{ 
  fprintf(stderr, "AirTraf: a wireless (802.11) network performance analyzer. by Peter K. Lee\n");
  fprintf(stderr, "<init-airtraf>: utility for initializing the database.\n");
  fprintf(stderr, "\nUsage:\n\t%s [-h <host>] [-u <user>] [-p <password>] [-P <port>] [-S <socket>] [-I <filename>] [-R] [-H] dbname\n\n", pname);
  fprintf(stderr, "-h : host to connect to *default:localhost*\n");
  fprintf(stderr, "-u : username to connect as *default:login*\n");
  fprintf(stderr, "-p : password *default:none* (optional)\n");
  fprintf(stderr, "-P : port *default:default port* \n");
  fprintf(stderr, "-S : socket *default:default socket* \n");
  fprintf(stderr, "-I : import sniff servers information (specify filename) *see db_import.cfg \n");
  fprintf(stderr, "-R : resets all database information (use with care) \n");
  fprintf(stderr, "-H : view this usage message\n\n");
  exit(1);
}

////////////////////////////////////////////////////////////////
//  RESET_DB() : clear all and destroy all data!!!
////////////////////////////////////////////////////////////////

// PROTOTYPES
void show_reset_warning();

/**
 * reset_db()
 * ----------
 * the function called to reset the named database.  Basically
 * destroys all tables, destroying all data in the process.  Not
 * generally recommended to be called, other than purposes of
 * troubleshooting errors that can't be easily fixed.
 **/
void reset_db(MYSQL *conn, char * db_name)
{
  char answer[20];
  char query[MAX_QUERY];
  
  int proceed = 0;
  
  /** show warning **/
  show_reset_warning();
  
  fprintf(stderr,"Destroying from database: %s\n",db_name);
  
  /** read input **/
  while (proceed == 0){
    fprintf(stderr,"Are you sure you want to proceed? (yes/no): ");
    scanf("%20s",answer);
    if(!strcasecmp("yes", answer)) {
      proceed = 1;
    } else if(!strcasecmp("no", answer)) {
      fprintf(stderr,"\nReset cancelled by user.\n");
      return;
    } else {
      proceed = 0;
    }
  }

  fprintf(stderr,"\nDestroying all from '%s'... ", db_name);

  /** construct query to drop tables **/
  snprintf(query, MAX_QUERY, "DROP TABLE IF EXISTS sniff_info, traffic_ap_5, traffic_node_5");

  /** issue query **/
  if (db_query(conn,query) < 0){
    fprintf(stderr,"ERROR\n");
    print_error(conn,"Problem dropping tables from Database");
  }
  else{
    fprintf(stderr,"OK\n\nDONE!\n");
  }
  
}

void show_reset_warning()
{
  fprintf(stderr,"\n*************************************************************\n");
  fprintf(stderr,"*WARNING: Resetting of database means you're going to destroy\n");
  fprintf(stderr," all traces of tables that previously occupied the database,\n");
  fprintf(stderr," meaning you're going to LOSE ALL THE DATA.\n\n");
  fprintf(stderr,"Please make sure this is EXACTLY what you want to do, and\n");
  fprintf(stderr,"even if you're sure, think again!\n\n");
  fprintf(stderr,"*NOTE: This should take place only if you purposely want to \n");
  fprintf(stderr," clear all traces of previous database information, i.e. \n");
  fprintf(stderr," if previous attempt at importing data failed, or there's \n");
  fprintf(stderr," irrevocable damage to the previous database.");
  fprintf(stderr,"\n*************************************************************\n\n");
}


////////////////////////////////////////////////////////////////
//  IMPORT_INTO_DB() : self-explanatory
////////////////////////////////////////////////////////////////

// PROTOTYPES for use by import_into_db()
void show_import_warning();
int check_if_comment(FILE *fp);
int skip_line(FILE *fp);
int parse_line(MYSQL * conn, FILE *fp);

void make_sniff_info(char * query);
void make_traffic_ap(char * query, int interval);
void make_traffic_node(char * query, int interval);

void import_into_db(MYSQL *conn, char *importfile)
{
  FILE *fp;
  //  MYSQL_RES *res_set;
  
  char answer[20];
  char query[MAX_QUERY];
  
  int proceed = 0;
  int status = 1;
  int exists;
  
  int tot_lines;
  int parsed_lines;
  
  /** show warning **/
  show_import_warning();
  
  fprintf(stderr,"Importing from file: %s\n",importfile);
  
  /** read input **/
  while (proceed == 0){
    fprintf(stderr,"Are you sure you want to proceed? (yes/no): ");
    scanf("%20s",answer);
    if(!strcasecmp("yes", answer)) {
      proceed = 1;
    } else if(!strcasecmp("no", answer)) {
      fprintf(stderr,"\nImport cancelled by user.\n");
      return;
    } else {
      proceed = 0;
    }
  }

  fprintf(stderr,"\nImporting initiated...\n");

  fprintf(stderr,"Checking if data structure already exists... ");
  /** do database query to check if table exists **/
  if (db_query(conn,"SELECT * FROM sniff_info") < 0){
    exists = 0;
  }
  else{
    exists = 1;
  }
  
  /** if it already exists exit **/
  if (exists){
    fprintf(stderr,"yes\n");
    fprintf(stderr,"No import will take place at this time.  Use web administration");
    fprintf(stderr," tool to make proper configuration changes.\n\n");
    return;
  }
  /** if it does not exist **/
  else{
    fprintf(stderr,"no\n");
    fprintf(stderr,"\nCreating Tables\n");

    /** put together the sniff_info query **/
    fprintf(stderr,"creating sniff_info... ");
    make_sniff_info(query);
    
    /** issue query **/
    if (db_query(conn,query) < 0){
      fprintf(stderr,"failed\n");
      print_error(conn,"Unable to Create Table");
      fprintf(stderr,"\nImport Database Failed!\n");
      return;
    }
    else{
      fprintf(stderr,"OK\n");
    }

    /** put together the traffic_ap_5 query **/
    fprintf(stderr,"creating traffic_ap_5... ");
    make_traffic_ap(query, 5);

    /** issue query **/
    if (db_query(conn,query) < 0){
      fprintf(stderr,"failed\n");
      print_error(conn,"Unable to Create Table");
      fprintf(stderr,"\nImport Database Failed!\n");
      return;
    }
    else{
      fprintf(stderr,"OK\n");
    }

    /** put together the traffic_node_5 query **/
    fprintf(stderr,"creating traffic_node_5... ");
    make_traffic_node(query, 5);

    /** issue query **/
    if (db_query(conn, query) < 0){
      fprintf(stderr,"failed\n");
      print_error(conn,"Unable to Create Table");
      fprintf(stderr,"\nImport Database Failed!\n");
      return;
    }
    else{
      fprintf(stderr,"OK\n");
    }
  }

  fprintf(stderr,"\nReading file '%s'...\n",importfile);
  /** open the import file **/
  if ((fp = fopen(importfile, "r")) == NULL){
    fprintf(stderr,"file not found!\n");
    return;
  }
  fprintf(stderr,"Processing file '%s'...\n",importfile);

  /** process the import file **/
  tot_lines = 1;
  parsed_lines = 0;
  while(status){
    status = check_if_comment(fp);
    if (status == -1) break;   // there was error checking or EOF
    if (status){               // found comment
      tot_lines++;
      status = skip_line(fp);
      if (status == -1) break; // there was error skipping or EOF
    }
    else{                      // found real data
      tot_lines++;
      status = parse_line(conn, fp);
      if (status == -1){       // there was error parsing data
	fprintf(stderr,"Error parsing line %d\n", tot_lines);
	fprintf(stderr,"\nImport Database Failed!\n");
	return;
      }
      parsed_lines++;
    }
  }
  fprintf(stderr,"%d lines read and processed\n",parsed_lines);
  fprintf(stderr,"\nCongratulations!  All your data has been successfully\n");
  fprintf(stderr,"inserted into the database!\n");
}

//////////////////////////////////////////////////////////
//  Helper Functions for import_into_db()
//////////////////////////////////////////////////////////

void show_import_warning()
{
  fprintf(stderr,"\n*************************************************************\n");
  fprintf(stderr,"*WARNING: This importing of data into the database\n");
  fprintf(stderr," may cause unwanted consequences to the data structure\n");
  fprintf(stderr," of the database, which may result in improper operations\n");
  fprintf(stderr," of AirTraf polling server.\n\n");
  fprintf(stderr,"Please double check to make sure the data from the import\n");
  fprintf(stderr,"file is correct beyond resonable doubt.\n");
  fprintf(stderr,"See 'db_import.cfg' for examples\n\n");
  fprintf(stderr,"*NOTE: This should take place only *once*.  If the database\n");
  fprintf(stderr," has been initialized with this data already, no update will\n");
  fprintf(stderr," take place.  Please use the web administration tool to make\n");
  fprintf(stderr," the proper configuration changes.");
  fprintf(stderr,"\n*************************************************************\n\n");
}

void make_sniff_info(char * query)
{
  memset(query, 0, MAX_QUERY);
  snprintf(query, MAX_QUERY, "CREATE TABLE sniff_info(");
  strcat(query, "id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT, ");
  strcat(query, "ip_address CHAR(20) NOT NULL, ");
  strcat(query, "sniffer_name CHAR(20) NOT NULL, ");
  strcat(query, "location CHAR(50), status CHAR(20), ");
  strcat(query, "last_up DATETIME NOT NULL, ");
  strcat(query, "PRIMARY KEY (id))");
}

void make_traffic_ap(char * query, int interval)
{
  memset(query, 0, MAX_QUERY);
  snprintf(query, MAX_QUERY, "CREATE TABLE traffic_ap_%d(", interval);
  strcat(query, "id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT, ");
  strcat(query, "sniff_server    CHAR(20) NOT NULL, ");
  strcat(query, "ap_bssid        CHAR(12) NOT NULL, ");
  strcat(query, "ap_channel      TINYINT UNSIGNED NOT NULL, ");
  strcat(query, "ap_ssid         CHAR(50), ");
  strcat(query, "timestamp       DATETIME     NOT NULL, ");
  strcat(query, "mgmt_count      INT UNSIGNED NOT NULL, ");
  strcat(query, "mgmt_byte       INT UNSIGNED NOT NULL, ");
  strcat(query, "mgmt_beacon     INT UNSIGNED NOT NULL, ");
  strcat(query, "mgmt_avg_band   DECIMAL(6,4) NOT NULL, ");
  strcat(query, "mgmt_high_band  DECIMAL(6,4) NOT NULL, ");
  strcat(query, "mgmt_low_band   DECIMAL(6,4) NOT NULL, ");
  strcat(query, "mgmt_curr_band  DECIMAL(6,4) NOT NULL, ");
  strcat(query, "ctrl_count      INT UNSIGNED NOT NULL, ");
  strcat(query, "ctrl_byte       INT UNSIGNED NOT NULL, ");
  strcat(query, "ctrl_ack        INT UNSIGNED NOT NULL, ");
  strcat(query, "ctrl_avg_band   DECIMAL(6,4) NOT NULL, ");
  strcat(query, "ctrl_high_band  DECIMAL(6,4) NOT NULL, ");
  strcat(query, "ctrl_low_band   DECIMAL(6,4) NOT NULL, ");  
  strcat(query, "ctrl_curr_band  DECIMAL(6,4) NOT NULL, ");
  strcat(query, "data_count      INT UNSIGNED NOT NULL, ");
  strcat(query, "data_byte       INT UNSIGNED NOT NULL, ");
  strcat(query, "data_ext_count  INT UNSIGNED NOT NULL, ");
  strcat(query, "data_ext_byte   INT UNSIGNED NOT NULL, ");
  strcat(query, "data_int_count  INT UNSIGNED NOT NULL, ");
  strcat(query, "data_int_byte   INT UNSIGNED NOT NULL, ");
  strcat(query, "data_avg_band   DECIMAL(6,4) NOT NULL, ");
  strcat(query, "data_high_band  DECIMAL(6,4) NOT NULL, ");
  strcat(query, "data_low_band   DECIMAL(6,4) NOT NULL, ");
  strcat(query, "data_curr_band  DECIMAL(6,4) NOT NULL, ");
  strcat(query, "overall_count   INT UNSIGNED NOT NULL, ");
  strcat(query, "overall_byte    INT UNSIGNED NOT NULL, ");
  strcat(query, "overall_corrupt INT UNSIGNED NOT NULL, ");
  strcat(query, "overall_nodes   INT UNSIGNED NOT NULL, ");
  strcat(query, "overall_avg_band   DECIMAL(6,4) NOT NULL, ");
  strcat(query, "overall_high_band  DECIMAL(6,4) NOT NULL, ");
  strcat(query, "overall_low_band   DECIMAL(6,4) NOT NULL, ");
  strcat(query, "overall_curr_band  DECIMAL(6,4) NOT NULL, ");
  strcat(query, "PRIMARY KEY (id))");
}

void make_traffic_node(char * query, int interval)
{
  memset(query, 0, MAX_QUERY);
  snprintf(query, MAX_QUERY, "CREATE TABLE traffic_node_%d(", interval);
  strcat(query, "id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT, ");
  strcat(query, "ap_bssid        CHAR(12)     NOT NULL, ");
  strcat(query, "timestamp       DATETIME     NOT NULL, ");
  strcat(query, "node_mac        CHAR(12)     NOT NULL, ");
  strcat(query, "type            CHAR(6)      NOT NULL, ");
  strcat(query, "incoming_count  INT UNSIGNED NOT NULL, ");
  strcat(query, "incoming_byte   INT UNSIGNED NOT NULL, ");
  strcat(query, "outgoing_count  INT UNSIGNED NOT NULL, ");
  strcat(query, "outgoing_byte   INT UNSIGNED NOT NULL, ");
  strcat(query, "signal_strength DECIMAL(6,4) NOT NULL, ");
  strcat(query, "node_avg_band   DECIMAL(6,4) NOT NULL, ");
  strcat(query, "node_high_band  DECIMAL(6,4) NOT NULL, ");
  strcat(query, "node_low_band   DECIMAL(6,4) NOT NULL, ");  
  strcat(query, "node_curr_band  DECIMAL(6,4) NOT NULL, ");
  strcat(query, "PRIMARY KEY (id))");
}

int parse_line(MYSQL * conn, FILE *fp)
{
  int result;
  char ip[20];
  char name[20];
  char loc[50];
  char query[MAX_QUERY];
  char temp[100];

  result = fscanf(fp,"%s%s%s",ip,name,loc);
  if ((result !=EOF) && (result == 3)){
    fprintf(stderr,"READ: ip= %s  name= %s  loc= %s\n",ip,name,loc);

    /** construct query **/
    snprintf(query, MAX_QUERY, "INSERT INTO sniff_info ");
    strcat(query, "(ip_address, sniffer_name, location) ");
    snprintf(temp, 100, "VALUES(\"%s\",\"%s\",\"%s\")", ip, name, loc);
    strcat(query, temp);
    
    if (db_query(conn,query) < 1){
      print_error(conn,"Unable to INSERT into db");
      return (-1);
    }
    else{
      fprintf(stderr,"INSERT into db: OK!\n");
      return (1);
    }
  }
  else{
    return (-1);
  }
}

int check_if_comment(FILE *fp)
{
  int result;
  char c;

  result = fscanf(fp,"%c",&c);
  if ((result != EOF) && (result > 0)){
    if ((c == '#') || (c == '\n')){
      return (1);
    }
    else{
      if (SEEK_CUR > SEEK_SET){
	result = fseek(fp,-1,SEEK_CUR);
      }
      return (0);
    }
  }
  else{
    return (-1);
  }  
}

int skip_line(FILE *fp)
{
  int result;
  char c;

  do{
    result = fscanf(fp, "%c",&c);    
  } while((c != '\n') && (result != EOF) && (result > 0));

  if (c == '\n'){
    return (1);
  }
  else{
    return (-1);
  }
}


/*=============================================================*/
/* Main Program */

int main(int argc, char **argv)
{
  /** program name **/
  char *pname = argv[0];
  
  char *host_name = def_host_name;
  char *user_name = def_user_name;
  char *password = def_password;
  unsigned int port_num = def_port_num;
  char *socket_name = def_socket_name;
  char *db_name = def_db_name;
  char *importfile = NULL;

  int mode = -1;
  
  char passbuf[100];
  char importbuf[100];
  char temp_db_name[20];
  
  int ask_password = 0;  
  int c, option_index = 0;
  
  if (argc < 2){
    usage(pname);
  }

  /** parse command-line arguments **/
  while ((c = getopt_long(argc, argv, "h:p::u:P:S:I:RH", long_options,
			   &option_index)) != EOF){
    switch(c)
      {
      case 'h':
	host_name = optarg;
	break;
      case 'u':
	user_name = optarg;
	break;
      case 'p':
	if (!optarg)
	  ask_password = 1;
	else{
	  (void) strncpy (passbuf, optarg, sizeof(passbuf) -1);
	  passbuf[sizeof(passbuf)-1] = '\0';
	  password = passbuf;
	  while (*optarg)
	    *optarg++ = ' ';
	}
	break;
      case 'P':
	port_num = (unsigned int) atoi (optarg);
	break;
      case 'S':
	socket_name = optarg;
	break;
      case 'I':
	mode = IMPORT;
	if (!optarg){
	  fprintf(stderr, "\nMust specify filename to import data from.  View 'sniff_info.example' for info.\n");
	  exit(1);
	}
	else{
	  (void) strncpy(importbuf, optarg, sizeof(importbuf) -1);
	  importbuf[sizeof(importbuf)-1] = '\0';
	  importfile = importbuf;
	  while (*optarg){
	    *optarg++ = ' ';
	  }
	}
	break;
      case 'R':
	mode = RESET;
	break;
      case 'H':
	usage(pname);
	break;
      case '?':
	fprintf(stderr,"\nInvalid option or missing parameter, use airtraf-p -H for help\n\n");
	exit(1);
      }
  }

  if (mode == -1){
    fprintf(stderr,"\nNo action specified...  choose to (import) or (reset)\n");
    exit(1);
  }

  argc -= optind; // advance past the arguments that were processed
  argv += optind; // by getopt_long()

  if (argc > 0){
    db_name = argv[0];
    --argc; ++argv;
  }
  else{
    sprintf(temp_db_name, "airtraf");
    db_name = temp_db_name;
  }

  if (ask_password){
    password = get_tty_password(NULL);
  }

  /** connect to the database **/
  conn = db_connect(host_name, user_name, password, db_name, port_num, socket_name, 0);
  if (conn == NULL){
    fprintf(stderr,"Could not connect to the database\n");
    exit(1);
  }
  else{
    fprintf(stderr,"Connection Successful!\n");
  }
  
  /** If importing initial data into the database **/
  if (mode == IMPORT){
    import_into_db(conn, importfile);
    db_disconnect(conn);
    exit(0);
  }
  /** If clearing out the database **/
  else if (mode == RESET){
    reset_db(conn, db_name);
    db_disconnect(conn);
    exit(0);
  }
  return (0);
}
