/*********************************************************************
 **
 ** AirTraf - Utility to measure traffic in the wireless segment of
 **           802.11b network.  Discovers connected nodes, then tracks
 **           incoming/outgoing traffic for each node.  Shows the
 **           current bandwidth usage, as well as total packet count,
 **           inc/outgoing for each node, and tracks specific
 **           management frames sent from the AP.
 **
 *********************************************************************
 *
 * There are many programs currently out there that does performance
 * monitoring, however, this program is unique in that it isolates the
 * traffic to nodes within the wireless segment, regardless of the
 * fact that the wireless segment is connected to the rest of the
 * network or not.
 *
 * It will currently be used to analyze packet information upto layer
 * 2 of the OSI model, however, it can be extended beyond if proven
 * desirable.
 *
 * This utility is written for use with IEEE 802.11 adapters based
 * on Intersil's PRISM II chipset (PCMCIA).  However currently, the
 * limitation of Prism2 drivers limit the sniffing of channel to
 * single channels.  It will also be equipped to work with Cisco
 * Aironet chipset, which offers an advantage of sniffing on all
 * channels at once (will be added later).
 *
 * The linux driver for these cards can be found on www.linux-wlan.com
 * It has been verified with a LinkSys WPC11 IEEE 802.11 adapter. 
 * 
 * Copyright (c)2001,2002 by Elixar.net  Durham, NC., all rights reserved.
 *
 * Comments/Bug reports should be sent to: Peter K. Lee (saint@elixar.net)
 * 
 *********************************************************************/

/*=============================================================*/
/* System Includes */

#include <ncurses.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <asm/types.h>
#include <sys/types.h>
#include <errno.h>
#include <features.h>
#include <pthread.h>
#include "getopt.h"

#include "definition.h"
#include "sniff_include.h"

/*=============================================================*/
/* Local Static Definitions */

#define def_card          AIRONET
#define def_interface     NULL
#define def_mode          INTERACTIVE
#define def_conn_mode     ADHOC_MODE
#define def_logfile       "log.connection"
#define def_signal_support DISABLED
#define def_logging_mode  DISABLED

/*=============================================================*/
/* Local Global Variables */

int sysexit = 0;

/*=============================================================*/
/* Local Structs */

struct option long_options[] =
  {
    {"card",           required_argument, NULL, 'C'},
    {"interface",      required_argument, NULL, 'I'},
    {"daemonized",     no_argument,       NULL, 'D'},
    {"server_mode",    no_argument,       NULL, 'S'},
    {"logfile",        optional_argument, NULL, 'L'},
    {"signal",         no_argument,       NULL, 'x'},
    {"help",           no_argument,       NULL, 'h'},
    {"debug",          no_argument,       NULL, 'z'},
    { 0, 0, 0, 0}
  };

/*=============================================================*/
/* Function Prototypes */


/*=============================================================*/
/* Function Definitions */

static void usage (const char *pname)
{
  fprintf(stderr, "--------------------------------------------------------------------------\n");
  fprintf(stderr, "AirTraf: a wireless (802.11) network performance analyzer. by Peter K. Lee\n");
  fprintf(stderr, "\nUsage:\n\t%s [-C <cardtype>] [-I <interface>] [-D] [-S] [-x] [-L <logfile>]\n", pname);
  fprintf(stderr, "\n-C : card type, either prism2 or aironet \n");
  fprintf(stderr, "-I : interface to listen on eth0. eth1, wlan0, etc. *required*\n");
  fprintf(stderr, "-D : runs the program in daemonized background *default:OFF* \n");
  fprintf(stderr, "-S : runs the program in server mode (allows connection from polling server) *default:OFF* \n");
  fprintf(stderr, "-L : enable logging mode *default:OFF (argument allows specification of log file name) \n");
  fprintf(stderr, "-x : enable signal strength *default:OFF (using patched drivers) \n");
  fprintf(stderr, "-h : view this usage message\n\n");
  exit(1);
}

void stop_signal ()
{
  if (!sysexit){
    sysexit = 1;
    ask_stop_sniffer_engine();
  }
}

/*=============================================================*/
/* Main Program */

int main(int argc, char **argv)
{
  struct SETTINGS mySettings;

  /* program name */
  char *pname = argv[0];
  char lognamebuf[100];
  
  int c, option_index = 0;

  /* command line stuff */
  mySettings.card_type      = 0;
  mySettings.interface      = def_interface;
  mySettings.runtime_mode   = def_mode;
  mySettings.conn_mode      = def_conn_mode;
  mySettings.logfile        = def_logfile;
  mySettings.logging_mode   = def_logging_mode;
  mySettings.signal_support = def_signal_support;
  mySettings.sniff_socket   = 0;

  if(argc < 2){
    usage(pname);
  }
  
  if (getenv("TERM") == NULL) {
    fprintf(stderr, "Your TERM variable is not set.\n");
    fprintf(stderr, "Please set it to an appropriate value.\n");
    exit(1);
  }
  /** load options from configuration file **/
  //  loadoptions(&options);

  /* parse command-line arguments */
  while ((c = getopt_long (argc, argv, "C:I:DSL::hzx", long_options,
			   &option_index)) != EOF)
    {
      switch(c)
	{
	case 'C':
	  if(!strcasecmp("prism2", optarg)) {
	    mySettings.card_type = PRISMII;
	  } else if(!strcasecmp("aironet", optarg)) {
	    mySettings.card_type = AIRONET;
	  } else {
	    fprintf(stderr,"Invalid card type chosen!  Valid types: 'prism2' or 'aironet' (default)\n\n");
	    exit(1);
	  }
	  break;
	case 'I':
	  mySettings.interface = optarg;
	  break;
	case 'D':
	  printf("set to daemonized mode...\n");
	  mySettings.runtime_mode = DAEMONIZED;
	  break;
	case 'S':
	  mySettings.conn_mode = SERVER_MODE;
	  break;
	case 'L':
	  mySettings.logging_mode = ENABLED;
	  /* if there's optional argument */
	  if (optarg){    
	    strncpy(lognamebuf, optarg, sizeof(lognamebuf) -1);
	    lognamebuf[sizeof(lognamebuf)-1] = '\0';
	    mySettings.logfile = lognamebuf;
	    while (*optarg){
	      *optarg++ = ' ';
	    }
	  }
	  break;
	case 'x':
	  mySettings.signal_support = ENABLED;
	  break;
	case 'h':
	  usage(pname);
	  break;
	case 'z':
	  break;
	case '?':
	  fprintf(stderr,"\nInvalid option or missing parameter, use airtraf -h for help\n\n");
	  exit(1);
	}
    }

  if (mySettings.card_type == 0){
    fprintf(stderr,"\nYou need to specify card type, see below:\n\n");
    usage(pname);
  }
  
  /* if interface was not specified alert user */
  if (mySettings.interface == NULL){
    fprintf(stderr,"\nYou need to specify which interface to listen on, see below:\n\n");
    usage(pname);
  }
  
  //  setpriority (PRIO_PROCESS, 0 , -10);
  signal (SIGINT, stop_signal);  

  fprintf(stderr,"-----------------------------------------\n");
  fprintf(stderr,"Airtraf %s (c)2001,2002 Elixar.net\n", VERSION_INFO);
  fprintf(stderr,"Mode: sniffing server\n");
  fprintf(stderr,"Author: Peter K. Lee  All Rights Reserved\n");
  fprintf(stderr,"-----------------------------------------\n");

  /** initialize airtraf as defined in runtime.c **/
  if (DEBUG) printf("airtraf initializing...\n");
  airtraf_init(&mySettings);

  /** begin airtraf as defined in runtime.c **/
  if (DEBUG) printf("airtraf starting up...\n");
  airtraf_start(&mySettings);
  
  /** stop airtraf as defined in runtime.c **/
  if (DEBUG) printf("airtraf exiting...\n");
  airtraf_end(&mySettings);

  return(0);
}
