/****************************************************************
 **
 **  AIRTRAF:
 **     a wireless (802.11) traffic/performance analyzer
 **
 **  server.c
 **
 ****************************************************************
 **
 **   Copyright (c) 2001 all rights reserved.
 **
 **   Author: Peter K. Lee <pkl@duke.edu>
 **
 ***************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <pthread.h>
#include <time.h>

#include "definition.h"
#include "sniff_include.h"

extern pthread_t thread;
extern pthread_cond_t start_cond;
extern pthread_mutex_t start_lock;
extern bss_t *bss_list;

#define MAX_CMD_SIZE 100

/*=============================================================*/
/* Function Prototypes */

int process_command(int, char *);
int issue_get_cmd(int, char *);
int send_bss_info(int);
int send_ids_info(int);

/*=============================================================*/
/* Function Definitions */

/**
 * process_command()
 * -----------------
 * a routine to parse the command sent by the polling server,
 * determine the action associated, and call appropriate action
 * functions.
 **/
int process_command(int fd, char * cmd)
{
  char action[10];
  char option[10];
  
  sscanf(cmd, "%s%s", action, option);

  if (!strcasecmp(action, "GET")){
    return issue_get_cmd(fd, option);
  }
  else if (!strcasecmp(action, "SYNCH")){
    return (0);
  }
  return (-1);
}

/**
 * issue_get_cmd()
 * ---------------
 * get the proper data requested by the GET command
 **/
int issue_get_cmd(int fd, char *option)
{
  if (!strcasecmp(option, "DATA")){
    return send_bss_info(fd);
  }
  if (!strcasecmp(option, "IDS")){
    return send_ids_info(fd);
  }
  return (-1);
}

/**
 * send_bss_info()
 * ---------------
 * send the bss_info requested from the polling server
 **/
int send_bss_info(int fd)
{
  char code[8];
  int tot_sent = 0;
  int sent;
  bss_t * temp;
  
  for (temp = bss_list; temp; temp = temp->next)
    {
      if ((sent = send(fd, temp, sizeof(bss_t), 0)) < 0){
	perror ("Send error\n");
	exit(0);
      }
      fprintf(stderr,"sent %d bytes...\n",sent);
      tot_sent += sent;
    }
  strncpy(code,"\r\n",8);
  if ((sent = send(fd, code, strlen(code), 0)) < 0){
    perror ("Send error\n");
    exit(0);
  }
  fprintf(stderr,"sent done code...\n");
  tot_sent += sent;
  return tot_sent;
}

int send_ids_info(int fd)
{
  return (-1);
}

//////////////////////////////////////////////////////////////////
// MAIN ROUTINE : SERVER
/////////////////////////////////////////////////////////////////

/**
 * server()
 * --------
 * the threaded routine that opens up a port (2222) and listens for
 * connections from polling server so that data may be transmitted.
 **/
void *server(void *var)
{
  int listenfd;
  int connfd;
  socklen_t len;
  struct sockaddr_in servaddr, clientaddr;

  unsigned char logmsg[MAX_MSG_SIZE];
  unsigned char tempmsg[100];
  char buff[MAX_BUFFER_SIZE];
  char cmd[MAX_CMD_SIZE];

  int result;
  int myset = 10; // socket option to reuse address
  int transmit = 0;

  struct SETTINGS *mySettings = (struct SETTINGS*)var;
  
  /* we want this server to be the only thing running */
  pthread_mutex_lock(&start_lock);

  if (( listenfd = socket(AF_INET, SOCK_STREAM, 0) ) < 0){
    perror ("Could not create server socket\n");
    exit(0);
  }

  /*
   * Set socket options to enable resueaddr.
   */
  if ((result = setsockopt (listenfd, SOL_SOCKET, SO_REUSEADDR, &myset, sizeof(myset))) < 0)
    {
      perror ("Could not set socket options");
      exit(0);
    }

  bzero(&servaddr, sizeof(servaddr));
  servaddr.sin_family = AF_INET;
  servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
  servaddr.sin_port = htons(2222);
  
  if (bind(listenfd, (struct sockaddr *) &servaddr, sizeof(servaddr)) < 0){
    perror ("Call to bind failed\n");
    exit(0);
  }

  pthread_cond_broadcast(&start_cond);
  pthread_mutex_unlock(&start_lock);

  if (mySettings->logging_mode == ENABLED){
    bzero(logmsg,sizeof(logmsg));  
    sprintf(logmsg,"** AirTraf Server Started,  Listening on port 2222 **\n");
    write_log(CONNECT_LOG,logmsg);    
  }
  
  if (listen(listenfd, SOMAXCONN) < 0){
    perror("Listen error\n");
    exit(0);
  }
  
  while(!sysexit){
    int port_num;
    int recvlen;

    len = sizeof(clientaddr);
    if ((connfd = accept(listenfd, (struct sockaddr *) &clientaddr, &len)) < 0){
      perror ("Accept error\n");
      exit(0);
    }
    port_num = ntohs(clientaddr.sin_port);

    if (mySettings->logging_mode == ENABLED){
      bzero(logmsg, sizeof(logmsg));
      bzero(tempmsg, sizeof(tempmsg));
      sprintf(tempmsg,"Connection from %s, port %d:: ",
	      inet_ntop(AF_INET, &clientaddr.sin_addr, buff, sizeof(buff)),port_num);
      strncat(logmsg, tempmsg, sizeof(tempmsg));
    }

    bzero(cmd, MAX_CMD_SIZE);
    fprintf(stderr,"receiving command... ");
    /** first receive command from polling server **/
    if ((recvlen = recv(connfd,cmd,sizeof(cmd),0))<=0){
      perror ("error in receive\n");
      return (NULL);
    }
    fprintf(stderr,"OK (%s)\n", cmd);

    /** see what potential structs have filtered... **/
    track_bad_data();
    
    /** process the request by poll server **/
    transmit = process_command(connfd, cmd);

    /** write this transaction to log **/
    if (mySettings->logging_mode == ENABLED){
      bzero(tempmsg,sizeof(tempmsg));
      sprintf(tempmsg,"(%d total bytes sent)\n",transmit);
      strncat(logmsg, tempmsg, sizeof(tempmsg));
      write_log(CONNECT_LOG,logmsg);	
    }

    /** reset potential structures **/
    fprintf(stderr,"resetting temporary structs... ");
    reset_potential_structs();
    fprintf(stderr,"ok\n");

    /** reset bss_list structure **/
    fprintf(stderr,"freeing space... ");
    reset_bss_list(bss_list);
    bss_list = NULL;
    fprintf(stderr,"ok\n");
    
    /** close connection **/    
    if (close(connfd) < 0){
      perror ("Close error");
      exit(0);
    }
    if (mySettings->logging_mode == ENABLED){
      flush_log(CONNECT_LOG);
    }
  }

  if (close(listenfd) < 0){
    perror ("Close error");
    exit(0);
  }
  
  pthread_exit(&thread);
}
